import logging
import os


class Options(object):
    service = ''
    agent_host = ''
    agent_port = 0
    log_level = logging.WARN

    def __init__(self, **kwds):
        if "INSTANA_DEV" in os.environ:
            self.log_level = logging.DEBUG

        self.__dict__.update(kwds)



from __future__ import print_function
import opentracing as ot
from instana import tracer, options
import opentracing.ext.tags as ext
import os


DJ_INSTANA_MIDDLEWARE = 'instana.django.InstanaMiddleware'


class InstanaMiddleware(object):
    """ Django Middleware to provide request tracing for Instana """
    def __init__(self, get_response):
        self.get_response = get_response
        opts = options.Options(service="Django")
        ot.global_tracer = tracer.InstanaTracer(opts)
        self

    def __call__(self, request):
        env = request.environ
        if 'HTTP_X_INSTANA_T' in env and 'HTTP_X_INSTANA_S' in env:
            ctx = ot.global_tracer.extract(ot.Format.HTTP_HEADERS, env)
            span = ot.global_tracer.start_span("django", child_of=ctx)
        else:
            span = ot.global_tracer.start_span("django")

        span.set_tag(ext.HTTP_URL, env['PATH_INFO'])
        span.set_tag("http.params", env['QUERY_STRING'])
        span.set_tag(ext.HTTP_METHOD, request.method)
        span.set_tag("http.host", env['HTTP_HOST'])

        response = self.get_response(request)

        if 500 <= response.status_code <= 511:
            span.set_tag("error", True)
            ec = span.tags.get('ec', 0)
            span.set_tag("ec", ec+1)

        span.set_tag(ext.HTTP_STATUS_CODE, response.status_code)
        ot.global_tracer.inject(span.context, ot.Format.HTTP_HEADERS, response)
        span.finish()
        return response


def hook(module):
    """ Hook method to install the Instana middleware into Django """
    if "INSTANA_DEV" in os.environ:
        print("==============================================================")
        print("Instana: Running django hook")
        print("==============================================================")

    if DJ_INSTANA_MIDDLEWARE in module.settings.MIDDLEWARE:
        return

    if type(module.settings.MIDDLEWARE) is tuple:
        module.settings.MIDDLEWARE = (
                DJ_INSTANA_MIDDLEWARE,) + module.settings.MIDDLEWARE
    elif type(module.settings.MIDDLEWARE) is list:
        module.settings.MIDDLEWARE = [
                DJ_INSTANA_MIDDLEWARE] + module.settings.MIDDLEWARE
    else:
        print("Instana: Couldn't add InstanaMiddleware to Django")



class InstanaSpan(object):
    t = 0
    p = None
    s = 0
    ts = 0
    ta = "py"
    d = 0
    n = None
    f = None
    ec = 0
    error = False
    data = None

    def __init__(self, **kwds):
        self.__dict__.update(kwds)


class Data(object):
    service = None
    http = None
    baggage = None
    custom = None
    sdk = None

    def __init__(self, **kwds):
        self.__dict__.update(kwds)


class HttpData(object):
    host = None
    url = None
    status = 0
    method = None

    def __init__(self, **kwds):
        self.__dict__.update(kwds)


class CustomData(object):
    tags = None
    logs = None

    def __init__(self, **kwds):
        self.__dict__.update(kwds)


class SDKData(object):
    name = None
    Type = None
    arguments = None
    Return = None
    custom = None

    def __init__(self, **kwds):
        self.__dict__.update(kwds)



import opentracing as ot
from instana import tracer, options
import os


def hook(module):
    """ Hook method to install the Instana middleware into Flask """
    if os.environ["AUTOWRAPT_BOOTSTRAP"] == "runtime":
        if "INSTANA_DEV" in os.environ:
            print("==========================================================")
            print("Instana: Running runtime hook")
            print("==========================================================")

        opts = options.Options()
        ot.global_tracer = tracer.InstanaTracer(opts)



from __future__ import print_function
from instana import wsgi
import wrapt
import os


def wrapper(wrapped, instance, args, kwargs):
    rv = wrapped(*args, **kwargs)
    instance.wsgi_app = wsgi.iWSGIMiddleware(instance.wsgi_app)
    return rv


def hook(module):
    """ Hook method to install the Instana middleware into Flask """
    if "INSTANA_DEV" in os.environ:
        print("==============================================================")
        print("Instana: Running flask hook")
        print("==============================================================")
    wrapt.wrap_function_wrapper('flask', 'Flask.__init__', wrapper)



AGENT_DISCOVERY_URL = "/com.instana.plugin.python.discovery"
AGENT_TRACES_URL = "/com.instana.plugin.python/traces."
AGENT_DATA_URL = "/com.instana.plugin.python."
AGENT_DEFAULT_HOST = "localhost"
AGENT_DEFAULT_PORT = 42699
AGENT_HEADER = "Instana Agent"



import instana.options as o
import instana.meter as m
import instana.agent as a
from instana import log


class Sensor(object):
    options = None
    meter = None
    service_name = None
    agent = None

    def __init__(self, options):
        self.set_options(options)
        log.init(options.log_level)
        self.configure_service_name()
        self.agent = a.Agent(self)
        self.meter = m.Meter(self)

        log.debug("initialized sensor")

    def set_options(self, options):
        self.options = options
        if not self.options:
            self.options = o.Options()

    def configure_service_name(self):
        if self.options:
            self.service_name = self.options.service

    def handle_fork(self):
        self.agent = a.Agent(self)
        self.meter = m.Meter(self)



import random
import os
import time
import struct
import binascii

import sys
if sys.version_info.major is 2:
    string_types = basestring
else:
    string_types = str

_rnd = random.Random()
_current_pid = 0


def generate_id():
    """ Generate a 64bit signed integer for use as a Span or Trace ID """
    global _current_pid

    pid = os.getpid()
    if (_current_pid != pid):
        _current_pid = pid
        _rnd.seed(int(1000000 * time.time()) ^ pid)
    return _rnd.randint(-9223372036854775808, 9223372036854775807)


def id_to_header(id):
    """ Convert a 64bit signed integer to an unsigned base 16 hex string """

    if not isinstance(id, int):
        return ""

    byteString = struct.pack('>q', id)
    return str(binascii.hexlify(byteString).decode('UTF-8').lstrip('0'))


def header_to_id(header):
    """ Convert an unsigned base 16 hex string into a 64bit signed integer """

    if not isinstance(header, string_types):
        return 0

    # Pad the header to 16 chars
    header = header.zfill(16)
    r = binascii.unhexlify(header)
    return struct.unpack('>q', r)[0]



"""
Instana sensor and tracer. It consists of two modules that can be used as entry points:

- sensor: activates the meter to collect and transmit all kind of built-in metrics
- tracer: OpenTracing tracer implementation. It implicitly activates the meter
"""

__author__ = 'Instana Inc.'
__copyright__ = 'Copyright 2017 Instana Inc.'
__credits__ = ['Pavlo Baron', 'Peter Giacomo Lombardo']
__license__ = 'MIT'
__version__ = '0.6.12'
__maintainer__ = 'Peter Giacomo Lombardo'
__email__ = 'peter.lombardo@instana.com'

__all__ = ['sensor', 'tracer']
